from werkzeug.security import generate_password_hash, check_password_hash
from flask_login import UserMixin
from . import db

# Usertable which also stores the users wishlist and cart as a string of item ids split up by spaces

class User(UserMixin, db.Model):
    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(16), index=True, unique=True)
    password_hash = db.Column(db.String(64))
    cart = db.Column(db.String)
    wishlist = db.Column(db.String)
    email = db.Column(db.String)
    mobile = db.Column(db.String)

    def set_password(self, password):
        self.password_hash = generate_password_hash(password)

    def verify_password(self, password):
        return check_password_hash(self.password_hash, password)

    def new_cart(self):
        self.cart = ''

    def new_wishlist(self):
        self.wishlist = ''

    def set_mobile(self, mobile):
        self.mobile = mobile

    def set_email(self, email):
        self.email = email

    @staticmethod
    def register(username, password, email, mobile):
        user = User(username=username)
        user.set_password(password)
        user.set_mobile(mobile)
        user.set_email(email)
        user.new_cart()
        user.new_wishlist()
        db.session.add(user)
        db.session.commit()
        return user
    def __repr__(self):
        return '<User {0}>'.format(self.username)

# stores all of the information for the projects
class product_table(db.Model):
    __tablename__ = 'products'
    product_id = db.Column(db.Integer, primary_key=True)
    product_name = db.Column(db.String)
    product_price = db.Column(db.Float)
    product_footprint = db.Column(db.Float)
    product_image = db.Column(db.String)
    product_brand = db.Column(db.String)
    product_type = db.Column(db.String)
    product_description = db.Column(db.String)

    @staticmethod
    def new_product(name_in, price_in, footprint_in, image_in, brand_in, type_in, description_in):
        db.session.add(product_table(product_name = name_in, product_price = price_in, product_footprint = footprint_in,
                                     product_image = image_in, product_brand = brand_in, product_type = type_in,
                                     product_description = description_in))
        db.session.commit()

# stores the reviews as well as the author of the review
class review_table(db.Model):
    __tablename__ = 'reviews'
    review_id = db.Column(db.Integer, primary_key=True)
    product_id = db.Column(db.Integer)
    review = db.Column(db.String)
    review_author = db.Column(db.String)

    @staticmethod
    def new_review(review_in, product_id_in, author_in):
        db.session.add(review_table(review = review_in, product_id = product_id_in, review_author = author_in))
        db.session.commit()
